import { useCallback, useEffect, useRef, useState } from "react";
import createGlobe, { type COBEOptions } from "cobe";
import { useSpring } from "react-spring";
import classnames from "classnames";
import type { ZodNumberCheck } from "astro/zod";

type Marker = {
  location: [number, number];
  size: number;
  info: string;
};

const initialMarkers: Marker[] = [
  { location: [40.7128, -74.006], size: 0.05, info: "New York City" },
  { location: [34.0522, -118.2437], size: 0.05, info: "Los Angeles" },
  { location: [51.5074, -0.1278], size: 0.05, info: "London" },
  { location: [35.6895, 139.6917], size: 0.05, info: "Tokyo" },
  { location: [48.8566, 2.3522], size: 0.05, info: "Paris" },
  { location: [55.7558, 37.6176], size: 0.05, info: "Moscow" },
  { location: [28.6139, 77.209], size: 0.05, info: "New Delhi" },
  { location: [19.076, 72.8777], size: 0.05, info: "Mumbai" },
  { location: [-33.8688, 151.2093], size: 0.05, info: "Sydney" },
  { location: [-23.5505, -46.6333], size: 0.05, info: "São Paulo" },
  { location: [31.2304, 121.4737], size: 0.05, info: "Shanghai" },
  { location: [39.9042, 116.4074], size: 0.05, info: "Beijing" },
  { location: [6.5244, 3.3792], size: 0.05, info: "Lagos" },
  { location: [-26.2041, 28.0473], size: 0.05, info: "Johannesburg" },
  { location: [41.9028, 12.4964], size: 0.05, info: "Rome" },
  { location: [52.52, 13.405], size: 0.05, info: "Berlin" },
  { location: [-34.6037, -58.3816], size: 0.05, info: "Buenos Aires" },
  { location: [37.7749, -122.4194], size: 0.05, info: "San Francisco" },
  { location: [43.6532, -79.3832], size: 0.05, info: "Toronto" },
  { location: [13.7563, 100.5018], size: 0.05, info: "Bangkok" },
  // Add more cities as needed
];

const GLOBE_CONFIG: COBEOptions = {
  width: 800,
  height: 800,
  onRender: () => {},
  devicePixelRatio: 2,
  phi: 0,
  theta: 0.3,
  dark: 0.9,
  diffuse: 1.5, // Increased for better lighting
  mapSamples: 16000,
  mapBrightness: 5.5, // Increased brightness
  baseColor: [0.11, 0.08, 0.38], // Darker base color for contrast UIColor(red: 0.12, green: 0.07, blue: 0.15, alpha: 1)
  markerColor: [0.8, 0.075, 0.49], // Changed to pink color for markers
  glowColor: [0.07, 0.09, 0.15], // No glow
  markers: initialMarkers,
};

export default function Globe({
  year,
  setYear,
  className,
  config = GLOBE_CONFIG,
}: {
  year: number;
  setYear: any;
  className?: string;
  config?: COBEOptions;
}) {
  const phiRef = useRef(0);
  const widthRef = useRef(0);

  const pointerInteracting = useRef<number | null>(null);
  const pointerInteractionMovement = useRef(0);

  const canvasRef = useRef<HTMLCanvasElement>(null);

  const [markers, setMarkers] = useState<Marker[]>(initialMarkers);

  const [{ r }, api] = useSpring(() => ({
    r: 0,
    config: {
      mass: 1,
      tension: 280,
      friction: 40,
      precision: 0.001,
    },
  }));
  const intervalRef = useRef<NodeJS.Timeout | null>(null);

  // // Automatically increment the year every second until 2050
  // useEffect(() => {
  //   // If the interval is already set, do nothing
  //   if (intervalRef.current) {
  //     return;
  //   }

  //   // Only set interval if year is less than 2050
  //   if (year < 2050) {
  //     intervalRef.current = setInterval(() => {
  //       setYear((prevYear) => {
  //         if (prevYear < 2050) {
  //           return prevYear + 1;
  //         } else {
  //           // Year has reached 2050, clear interval
  //           clearInterval(intervalRef.current!);
  //           intervalRef.current = null;
  //           return prevYear;
  //         }
  //       });
  //     }, 1000);
  //   }

  //   // Clean up on unmount or when year changes
  //   return () => {
  //     if (intervalRef.current) {
  //       clearInterval(intervalRef.current);
  //       intervalRef.current = null;
  //     }
  //   };
  // }, [year]);

  // Update markers whenever the year changes
  useEffect(() => {
    setMarkers((prevMarkers) =>
      prevMarkers.map((marker) => ({
        ...marker,
        size: computeMarkerSize(year),
      }))
    );
  }, [year]);

  // Function to compute marker size based on the year
  function computeMarkerSize(year: number) {
    const baseSize = 0.05;
    const sizeIncrement = 0.005; // Adjust this value to control growth rate
    const yearsSince2020 = year - 2020;
    return baseSize + sizeIncrement * yearsSince2020;
  }

  // Function to compute deaths based on the year
  function computeDeaths(year: number): number {
    const startDeaths = 1.5e6; // 1.5 million
    const endDeaths = 10e6; // 40 million
    const startYear = 2020;
    const endYear = 2050;
    const deathsPerYear = (endDeaths - startDeaths) / (endYear - startYear);
    return startDeaths + (year - startYear) * deathsPerYear;
  }

  const deaths = computeDeaths(year);
  const deathsInMillions = (deaths / 1e6).toFixed(1);

  const updatePointerInteraction = (value: number | null) => {
    pointerInteracting.current = value;
    if (canvasRef.current) {
      canvasRef.current.style.cursor = value ? "grabbing" : "grab";
    }
  };

  const updateMovement = (clientX: number) => {
    if (pointerInteracting.current !== null) {
      const delta = clientX - pointerInteracting.current;
      pointerInteractionMovement.current = delta;
      api.start({ r: delta / 200 });
    }
  };

  const onRenderRef = useRef<(state: any) => void>();

  useEffect(() => {
    onRenderRef.current = (state: Record<string, any>) => {
      if (!pointerInteracting.current) phiRef.current += 0.005;
      state.phi = phiRef.current + r.get();
      state.width = widthRef.current * 2;
      state.height = widthRef.current * 2;
      state.markers = markers;
    };
  }, [markers, pointerInteracting, r]);

  const onResize = () => {
    if (canvasRef.current) {
      widthRef.current = canvasRef.current.offsetWidth;
    }
  };

  useEffect(() => {
    window.addEventListener("resize", onResize);
    onResize();

    if (!canvasRef.current || widthRef.current === 0) return;

    const globe = createGlobe(canvasRef.current!, {
      ...config,
      width: widthRef.current * 2,
      height: widthRef.current * 2,
      onRender: (state) => {
        if (onRenderRef.current) {
          onRenderRef.current(state);
        }
      },
    });

    setTimeout(() => {
      if (canvasRef.current) {
        canvasRef.current.style.opacity = "1";
      }
    });

    return () => {
      globe.destroy();
      window.removeEventListener("resize", onResize);
    };
  }, [config, widthRef.current]);

  // Updated slider styles to match the pink theme
  const customSliderStyles = `
    input[type='range'] {
      cursor: pointer;
      appearance: none;
      width: 100%;
      background: transparent;
    }

    /* WebKit */
    input[type='range']::-webkit-slider-thumb {
      height: 10px;
      width: 10px;
      background: #FE2CA6; /* Pink color */
      border-radius: 50%;
      cursor: pointer;
      -webkit-appearance: none;
      margin-top: -2px; /* Center the thumb */
    }

    input[type='range']::-webkit-slider-runnable-track {
      height: 6px;
      background: linear-gradient(
        to right,
        #FE2CA6 0%,
        #FE2CA6 ${((year - 2020) / (2050 - 2020)) * 100}%,
        #64748b ${((year - 2020) / (2050 - 2020)) * 100}%,
        #64748b 100%
      );
      border-radius: 5px;
    }

    /* Mozilla */
    input[type='range']::-moz-range-thumb {
      height: 10px;
      width: 10px;
      background: #FE2CA6; /* Pink color */
      border-radius: 50%;
      cursor: pointer;
    }

    input[type='range']::-moz-range-track {
      height: 6px;
      background: linear-gradient(
        to right,
        #FE2CA6 0%,
        #FE2CA6 ${((year - 2020) / (2050 - 2020)) * 100}%,
        #64748b ${((year - 2020) / (2050 - 2020)) * 100}%,
        #64748b 100%
      );
      border-radius: 5px;
    }

    /* Internet Explorer */
    input[type='range']::-ms-thumb {
      height: 10px;
      width: 10px;
      background: #FE2CA6; /* Pink color */
      border-radius: 50%;
      cursor: pointer;
    }

    input[type='range']::-ms-track {
      height: 6px;
      background: transparent;
      border-color: transparent;
      color: transparent;
    }

    input[type='range']::-ms-fill-lower {
      background: #FE2CA6; /* Pink color */
      border-radius: 5px;
    }

    input[type='range']::-ms-fill-upper {
      background: #64748b;
      border-radius: 5px;
    }
  `;

  return (
    <div
      className={classnames(
        "inset-0 mx-auto aspect-[1/1] w-full max-w-[600px]",
        className
      )}
    >
      <style>{customSliderStyles}</style>
      <canvas
        className={classnames(
          "h-full w-full opacity-0 transition-opacity duration-500 [contain:layout_paint_size]"
        )}
        ref={canvasRef}
        onPointerDown={(e) =>
          updatePointerInteraction(
            e.clientX - pointerInteractionMovement.current
          )
        }
        onPointerUp={() => updatePointerInteraction(null)}
        onPointerOut={() => updatePointerInteraction(null)}
        onMouseMove={(e) => updateMovement(e.clientX)}
        onTouchMove={(e) =>
          e.touches[0] && updateMovement(e.touches[0].clientX)
        }
      />
      <div className="absolute bottom-[-50px] left-[30px] sm:left-[60px] p-5 bg-gray-1 text-white rounded-lg max-w-full">
        <div className="flex flex-col items-start w-full">
          <span className="text-lg md:text-2xl font-medium w-full text-gray-11">
            <span className="text-xl md:text-2xl text-white-1 font-bold">
              {deathsInMillions} million{" "}
            </span>
            deaths annually
          </span>
          <span className="text-lg mt-1 w-full font-roboto-mono text-[#FE2CA6]">
            {Math.round(year)}
          </span>
          <div className="mt-1 w-full">
            <input
              type="range"
              min="2020"
              max="2050"
              value={year}
              onChange={(e) => setYear(parseInt(e.target.value, 10))}
              className="range-slider w-full cursor-pointer appearance-none bg-gray-6 rounded-md"
            />
            <div className="flex justify-between text-sm mt-1 w-full space-x-2 md:space-x-20">
              <span>Now</span>
              <span>2030</span>
              <span>2040</span>
              <span>2050</span>
            </div>
          </div>
        </div>
      </div>
    </div>
  );
}
